@@ -35,63 +35,79 @@ module Network.TLS.Extra.Cipher
     , cipher_DHE_DSS_RC4_SHA1
     , cipher_DHE_RSA_AES128GCM_SHA256
     , cipher_ECDHE_RSA_AES128GCM_SHA256
+    , cipher_ECDHE_RSA_AES128CBC_SHA256
+    , cipher_ECDHE_RSA_AES128CBC_SHA
+    , cipher_ECDHE_ECDSA_AES128GCM_SHA256
     ) where
 
 import qualified Data.ByteString as B
 
 import Network.TLS (Version(..))
 import Network.TLS.Cipher
-import qualified "cipher-rc4" Crypto.Cipher.RC4 as RC4
+import Data.Tuple (swap)
 
-import qualified "cipher-aes" Crypto.Cipher.AES as AES
+import Crypto.Cipher.AES
+import qualified Crypto.Cipher.RC4 as RC4
 import Crypto.Cipher.TripleDES
-import Crypto.Cipher.Types (makeKey, makeIV, cipherInit, cbcEncrypt, cbcDecrypt)
-import qualified Crypto.Cipher.Types as T
-
+import Crypto.Cipher.Types hiding (Cipher, cipherName)
+import Crypto.Error
 
 takelast :: Int -> B.ByteString -> B.ByteString
 takelast i b = B.drop (B.length b - i) b
 
 aes128cbc :: BulkDirection -> BulkKey -> BulkBlock
 aes128cbc BulkEncrypt key =
-    let ctx = AES.initAES key
-     in (\iv input -> let output = AES.encryptCBC ctx iv input in (output, takelast 16 output))
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\iv input -> let output = cbcEncrypt ctx (makeIV_ iv) input in (output, takelast 16 output))
 aes128cbc BulkDecrypt key =
-    let ctx = AES.initAES key
-     in (\iv input -> let output = AES.decryptCBC ctx iv input in (output, takelast 16 input))
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\iv input -> let output = cbcDecrypt ctx (makeIV_ iv) input in (output, takelast 16 input))
 
 aes256cbc :: BulkDirection -> BulkKey -> BulkBlock
 aes256cbc BulkEncrypt key =
-    let ctx = AES.initAES key
-     in (\iv input -> let output = AES.encryptCBC ctx iv input in (output, takelast 16 output))
+    let ctx = noFail (cipherInit key) :: AES256
+     in (\iv input -> let output = cbcEncrypt ctx (makeIV_ iv) input in (output, takelast 16 output))
 aes256cbc BulkDecrypt key =
-    let ctx = AES.initAES key
-     in (\iv input -> let output = AES.decryptCBC ctx iv input in (output, takelast 16 input))
+    let ctx = noFail (cipherInit key) :: AES256
+     in (\iv input -> let output = cbcDecrypt ctx (makeIV_ iv) input in (output, takelast 16 input))
 
 aes128gcm :: BulkDirection -> BulkKey -> BulkAEAD
 aes128gcm BulkEncrypt key =
-    let ctx = AES.initAES key
-     in (\nonce d ad -> AES.encryptGCM ctx nonce ad d)
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\nonce d ad ->
+            let aeadIni = noFail (aeadInit AEAD_GCM ctx nonce)
+             in swap $ aeadSimpleEncrypt aeadIni ad d 16)
 aes128gcm BulkDecrypt key =
-    let ctx = AES.initAES key
-     in (\nonce d ad -> AES.decryptGCM ctx nonce ad d)
+    let ctx = noFail (cipherInit key) :: AES128
+     in (\nonce d ad ->
+            let aeadIni = noFail (aeadInit AEAD_GCM ctx nonce)
+             in simpleDecrypt aeadIni ad d)
+  where
+    simpleDecrypt aeadIni header input = (output, tag)
+      where
+            aead                = aeadAppendHeader aeadIni header
+            (output, aeadFinal) = aeadDecrypt aead input
+            tag                 = aeadFinalize aeadFinal 16
+
+noFail :: CryptoFailable a -> a
+noFail = throwCryptoError
+
+makeIV_ :: BlockCipher a => B.ByteString -> IV a
+makeIV_ = maybe (error "makeIV_") id . makeIV
 
 tripledes_ede :: BulkDirection -> BulkKey -> BulkBlock
 tripledes_ede BulkEncrypt key =
-    let ctx = cipherInit (tripledes_key key)
-     in (\iv input -> let output = cbcEncrypt ctx (tripledes_iv iv) input in (output, takelast 16 output))
+    let ctx = noFail $ cipherInit key
+     in (\iv input -> let output = cbcEncrypt ctx (tripledes_iv iv) input in (output, takelast 8 output))
 tripledes_ede BulkDecrypt key =
-    let ctx = cipherInit (tripledes_key key)
-     in (\iv input -> let output = cbcDecrypt ctx (tripledes_iv iv) input in (output, takelast 16 input))
+    let ctx = noFail $ cipherInit key
+     in (\iv input -> let output = cbcDecrypt ctx (tripledes_iv iv) input in (output, takelast 8 input))
 
-tripledes_key :: BulkKey -> T.Key DES_EDE3
-tripledes_key key = either (\ke -> error ("tripledes cipher key internal error: " ++ show ke)) id $ makeKey key
-
-tripledes_iv :: BulkIV -> T.IV DES_EDE3
+tripledes_iv :: BulkIV -> IV DES_EDE3
 tripledes_iv iv = maybe (error "tripledes cipher iv internal error") id $ makeIV iv
 
 rc4 :: BulkDirection -> BulkKey -> BulkStream
-rc4 _ bulkKey = BulkStream (combineRC4 $ RC4.initCtx bulkKey)
+rc4 _ bulkKey = BulkStream (combineRC4 $ RC4.initialize bulkKey)
   where
     combineRC4 ctx input =
         let (ctx', output) = RC4.combine ctx input
@@ -102,7 +118,9 @@ rc4 _ bulkKey = BulkStream (combineRC4 $ RC4.initCtx bulkKey)
 -- this choice of ciphersuite should satisfy most normal need
 ciphersuite_all :: [Cipher]
 ciphersuite_all =
-    [ cipher_DHE_RSA_AES256_SHA256, cipher_DHE_RSA_AES128_SHA256
+    [ cipher_ECDHE_RSA_AES128GCM_SHA256
+    , cipher_ECDHE_ECDSA_AES128GCM_SHA256
+    , cipher_DHE_RSA_AES256_SHA256, cipher_DHE_RSA_AES128_SHA256
     , cipher_DHE_RSA_AES256_SHA1, cipher_DHE_RSA_AES128_SHA1
     , cipher_DHE_DSS_AES256_SHA1, cipher_DHE_DSS_AES128_SHA1
     , cipher_AES128_SHA256, cipher_AES256_SHA256
@@ -118,7 +136,7 @@ ciphersuite_medium = [cipher_RC4_128_MD5, cipher_RC4_128_SHA1, cipher_AES128_SHA
 
 -- | the strongest ciphers supported.
 ciphersuite_strong :: [Cipher]
-ciphersuite_strong = [cipher_DHE_RSA_AES256_SHA256, cipher_AES256_SHA256, cipher_AES256_SHA1]
+ciphersuite_strong = [cipher_ECDHE_RSA_AES128GCM_SHA256, cipher_ECDHE_ECDSA_AES128GCM_SHA256, cipher_DHE_RSA_AES256_SHA256, cipher_AES256_SHA256, cipher_AES256_SHA1]
 
 -- | DHE-RSA cipher suite
 ciphersuite_dhe_rsa :: [Cipher]
@@ -138,6 +156,8 @@ bulk_null = Bulk
     { bulkName         = "null"
     , bulkKeySize      = 0
     , bulkIVSize       = 0
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 0
     , bulkF            = BulkStreamF passThrough
     }
@@ -148,6 +168,8 @@ bulk_rc4 = Bulk
     { bulkName         = "RC4-128"
     , bulkKeySize      = 16
     , bulkIVSize       = 0
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 0
     , bulkF            = BulkStreamF rc4
     }
@@ -156,6 +178,8 @@ bulk_aes128 = Bulk
     { bulkName         = "AES128"
     , bulkKeySize      = 16
     , bulkIVSize       = 16
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 16
     , bulkF            = BulkBlockF aes128cbc
     }
@@ -164,6 +188,8 @@ bulk_aes128gcm = Bulk
     { bulkName         = "AES128GCM"
     , bulkKeySize      = 16 -- RFC 5116 Sec 5.1: K_LEN
     , bulkIVSize       = 4  -- RFC 5288 GCMNonce.salt, fixed_iv_length
+    , bulkExplicitIV   = 8
+    , bulkAuthTagLen   = 16
     , bulkBlockSize    = 0  -- dummy, not used
     , bulkF            = BulkAeadF aes128gcm
     }
@@ -172,6 +198,8 @@ bulk_aes256 = Bulk
     { bulkName         = "AES256"
     , bulkKeySize      = 32
     , bulkIVSize       = 16
+    , bulkExplicitIV   = 0
+    , bulkAuthTagLen   = 0
     , bulkBlockSize    = 16
     , bulkF            = BulkBlockF aes256cbc
     }
@@ -180,6 +208,8 @@ bulk_tripledes_ede = Bulk
     { bulkName      = "3DES-EDE-CBC"
     , bulkKeySize   = 24
     , bulkIVSize    = 8
+    , bulkExplicitIV = 0
+    , bulkAuthTagLen = 0
     , bulkBlockSize = 8
     , bulkF         = BulkBlockF tripledes_ede
     }
@@ -232,7 +262,7 @@ cipher_RC4_128_SHA1 = Cipher
 cipher_AES128_SHA1 :: Cipher
 cipher_AES128_SHA1 = Cipher
     { cipherID           = 0x2f
-    , cipherName         = "RSA-aes128-sha1"
+    , cipherName         = "RSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
@@ -243,7 +273,7 @@ cipher_AES128_SHA1 = Cipher
 cipher_AES256_SHA1 :: Cipher
 cipher_AES256_SHA1 = Cipher
     { cipherID           = 0x35
-    , cipherName         = "RSA-aes256-sha1"
+    , cipherName         = "RSA-AES256-SHA1"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA1
     , cipherKeyExchange  = CipherKeyExchange_RSA
@@ -254,7 +284,7 @@ cipher_AES256_SHA1 = Cipher
 cipher_AES128_SHA256 :: Cipher
 cipher_AES128_SHA256 = Cipher
     { cipherID           = 0x3c
-    , cipherName         = "RSA-aes128-sha256"
+    , cipherName         = "RSA-AES128-SHA256"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_RSA
@@ -265,7 +295,7 @@ cipher_AES128_SHA256 = Cipher
 cipher_AES256_SHA256 :: Cipher
 cipher_AES256_SHA256 = Cipher
     { cipherID           = 0x3d
-    , cipherName         = "RSA-aes256-sha256"
+    , cipherName         = "RSA-AES256-SHA256"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA256
     , cipherKeyExchange  = CipherKeyExchange_RSA
@@ -363,6 +393,36 @@ cipher_ECDHE_RSA_AES128GCM_SHA256 = Cipher
     , cipherMinVer       = Just TLS12 -- RFC 5288 Sec 4
     }
 
+cipher_ECDHE_RSA_AES128CBC_SHA :: Cipher
+cipher_ECDHE_RSA_AES128CBC_SHA = Cipher
+    { cipherID           = 0xc013
+    , cipherName         = "ECDHE-RSA-AES128CBC-SHA"
+    , cipherBulk         = bulk_aes128
+    , cipherHash         = SHA1
+    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
+    , cipherMinVer       = Just TLS10
+    }
+
+--TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256 
+cipher_ECDHE_RSA_AES128CBC_SHA256 :: Cipher
+cipher_ECDHE_RSA_AES128CBC_SHA256 = Cipher
+    { cipherID           = 0xc027
+    , cipherName         = "ECDHE-RSA-AES128CBC-SHA"
+    , cipherBulk         = bulk_aes128
+    , cipherHash         = SHA256
+    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
+    , cipherMinVer       = Just TLS12 -- RFC 5288 Sec 4
+    }
+
+cipher_ECDHE_ECDSA_AES128GCM_SHA256 :: Cipher
+cipher_ECDHE_ECDSA_AES128GCM_SHA256 = Cipher
+    { cipherID           = 0xc02b
+    , cipherName         = "ECDHE-ECDSA-AES128GCM-SHA256"
+    , cipherBulk         = bulk_aes128gcm
+    , cipherHash         = SHA256
+    , cipherKeyExchange  = CipherKeyExchange_ECDHE_ECDSA
+    , cipherMinVer       = Just TLS12 -- RFC 5289
+    }
 {-
 TLS 1.0 ciphers definition
 
